import logging
import sys
import subprocess
from typing import List
import pathlib

import esmvalcore.experimental as esmvaltool
from esmvalcore.dataset import Dataset
from esmvalcore.local import LocalFile

ENCODING_STDOUT = sys.stdout.encoding or "utf-8"

BTF_SBATCH = str(pathlib.Path(__file__).parent / 'bft_sbatch.sh')

# TODO allow setting different config files in the same way that
# ESMValTool does using an argument parser

# TODO setting logger default level from command line
logging.captureWarnings(True)
logger = logging.getLogger(__name__)
logger.setLevel(logging.INFO)

def send_job(files: List[str]):
    """Send sbatch job to bring this files from tapes"""
    # TODO it would be nice to be able to configure the sbatch from
    # argument line
    if not files:
        return

    logger.info("Sending files: %s", files)
    subprocess.run(["sbatch", BTF_SBATCH] + files)

def get_size(file: LocalFile) -> int:
    """
    Get the size of a file using du with a subprocess.
    The methods from pathlib.Path and os, unfortunatelly doesn't return
    0 if the data is on tapes.
    """
    out = subprocess.run(["du", str(file)], capture_output=True)
    stdout = out.stdout.decode(ENCODING_STDOUT)
    return int(stdout.split("\t")[0])

def bring_from_tapes(recipe_file):
    recipe = esmvaltool.get_recipe(recipe_file)
    datasets = recipe.data.get('datasets', [])

    # TODO ideally should check if files are required in more than one
    # diag to avoid sending a request twice
    # Loop over diagnostics
    for diag in recipe.data['diagnostics'].values():
        if 'variables' not in diag or diag['variables'] is None:
            # Some diagnostics may not have variables defined,
            # in that case move to the next
            continue
        # Loop over variables in a diagnostic
        for var, var_values in diag['variables'].items():
            if 'short_name' not in var_values:
                # Add short_name if not given
                var_values['short_name'] = var
            # Define the target datasets (main + additional)
            var_datasets = datasets + var_values.get('additional_datasets', [])
            # Loop over target datasets
            for ds_values in var_datasets:
                # Define the dataset object
                ds_obj = Dataset(**var_values, **ds_values)
                # find the files
                ds_obj.find_files()
                # Get only LocalFiles that have size = 0 as str
                files = [
                    str(file) for file in ds_obj.files
                    if isinstance(file, LocalFile) and get_size(file) == 0
                ]
                send_job(files)

if __name__ == '__main__':
    bring_from_tapes(sys.argv[1])
